import {Paho} from 'ng2-mqtt/mqttws31';
let fieldSize = 20;

let snakeField:HTMLElement = <HTMLElement> document.getElementById("snakeField");
let scoreField:HTMLElement = <HTMLElement> document.getElementById("score");
let highscoreField:HTMLElement = <HTMLElement> document.getElementById("highscore");
let snake:number[][] = [[10, 10], [10, 11], [10, 12]];
let snakedir:number = 2;
let target:number[] = []
let score:number = 0;
let highscore:number = Number(localStorage.getItem("highscore"));
let firstButtonPressed:boolean = false;

function getRandomInt(max:number) {
    return Math.floor(Math.random() * Math.floor(max)) + 1;
}

function drawField(){
    let fieldHTML:string = "";
    for(let i:number = 1; i <= fieldSize; i++){
        for(let x:number = 1; x <= fieldSize; x++){
            let elementClass:string = "grid";
            for(let y:number = 0; y<snake.length; y++){
                let currentSnake = snake[y];
                if(currentSnake[0] == i && currentSnake[1] == x){
                    if(y == snake.length - 1)
                        elementClass += " snake-head";
                    else
                        elementClass += " snake-body";
                }
            }

            if(target[0] == i && target[1] == x)
                elementClass += " target";

            fieldHTML += `<span id="field-${i}-${x}" class="${elementClass}"></span>`;
        }
        if(i < fieldSize) {
            fieldHTML += "<br>"
        }
    }
    snakeField.innerHTML = fieldHTML;
}

function moveSnake(){
    let head:number[] = snake[snake.length - 1].slice(0);
    switch (snakedir){
        case 0:
            head[0]++;
            break;
        case 1:
            head[0]--;
            break;
        case 2:
            head[1]++;
            break;
        case 3:
            head[1]--;
            break;
    }
    if (!(snake.length - 3 < score))
        snake.shift();
    snake.push(head);
}

function detectEnd(){
    let gameFinished:boolean = false;
    for(let i:number = 0; i < snake.length; i++){
        let snake1:number[] = snake[i];
        for (let x:number = i + 1; x < snake.length; x++){
            let snake2:number[] = snake[x]
            if(snake1[0] == snake2[0] && snake1[1] == snake2[1])
                gameFinished = true
        }
        if(snake1[0] < 0 || snake1[0] > fieldSize || snake1[1] < 0 || snake1[1] > fieldSize)
            gameFinished = true
    }

    if(gameFinished){
        if(highscore < score){
            localStorage.setItem("highscore", String(score));
            alert("Game Over, your score: " + score + "\n This is your new highscore!!!");
        } else {
            alert("Game Over, your score: " + score);
        }
        clearInterval(mainLoop);
        location.reload();
    }
}

let mainLoop = window.setInterval(function(){
    if(firstButtonPressed) {
        for (let y: number = 0; y < snake.length; y++) {
            let currentSnake = snake[y];
            if (currentSnake[0] == target[0] && currentSnake[1] == target[1]) {
                score++;
                scoreField.innerText = String(score);
                target = [getRandomInt(fieldSize), getRandomInt(fieldSize)];
            }
        }
        moveSnake();
        detectEnd()
        drawField();
    }
}, 600);

//keypress handler
document.addEventListener('keydown', function(event) {
    firstButtonPressed = true;
    const key:string = event.key;
    switch (key){
        case "ArrowRight":
            if(snakedir != 3)
                snakedir = 2
            break;
        case "ArrowLeft":
            if(snakedir != 2)
                snakedir = 3;
            break;
        case "ArrowUp":
            if(snakedir != 0)
                snakedir = 1;
            break;
        case "ArrowDown":
            if(snakedir != 1)
                snakedir = 0
    }
});

highscoreField.innerText = String(highscore);
target = [getRandomInt(fieldSize), getRandomInt(fieldSize)];
drawField();