import Swal from 'sweetalert2';
import {Paho} from 'ng2-mqtt/mqttws31';

let fieldSize = 20;

let snakeField:HTMLElement = <HTMLElement> document.getElementById("snakeField");
let scoreField:HTMLElement = <HTMLElement> document.getElementById("score");
let highscoreField:HTMLElement = <HTMLElement> document.getElementById("highscore");
let partnerScoreField:HTMLElement = <HTMLElement> document.getElementById("partnerScore");
let newGameButton:HTMLElement = <HTMLElement> document.getElementById("newMultiplayerGame");
let snake:number[][] = [];
let snakePartner:number[][] = [];
let snakedir:number = 2;
let target:number[] = []
let score:number = 0;
let highscore:number = Number(localStorage.getItem("highscore"));
let firstButtonPressed:boolean = false;
let client:any;
let gameID:string = "";
let gameMaster:boolean = false;
let clientID:string = "";
let gameTopic:string = "";
let gameAccepted = false;

function getRandomInt(max:number) {
    return Math.floor(Math.random() * Math.floor(max)) + 1;
}

function drawField(){
    let fieldHTML:string = "";
    for(let i:number = 1; i <= fieldSize; i++){
        for(let x:number = 1; x <= fieldSize; x++){
            let elementClass:string = "drawGrid";
            for(let y:number = 0; y<snake.length; y++){
                let currentSnake:number[] = snake[y];
                if(currentSnake[0] == i && currentSnake[1] == x){
                    if(y == snake.length - 1)
                        elementClass += " snake-head";
                    else
                        elementClass += " snake-body";
                }
            }
            for(let y:number = 0; y<snakePartner.length; y++){
                let currentSnake:number[] = snakePartner[y];
                if(currentSnake[0] == i && currentSnake[1] == x){
                    if(y == snakePartner.length - 1)
                        elementClass += " snake-partner-head";
                    else
                        elementClass += " snake-partner-body";
                }
            }

            if(target[0] == i && target[1] == x)
                elementClass += " target";

            fieldHTML += `<span class="grid"><span id="field-${i}-${x}" class="${elementClass}"></span></span>`;
        }
        if(i < fieldSize) {
            fieldHTML += "<br>"
        }
    }
    snakeField.innerHTML = fieldHTML;
}

function moveSnake(){
    let head:number[] = snake[snake.length - 1].slice(0);
    switch (snakedir){
        case 0:
            head[0]++;
            break;
        case 1:
            head[0]--;
            break;
        case 2:
            head[1]++;
            break;
        case 3:
            head[1]--;
            break;
    }
    if (!(snake.length - 3 < score))
        snake.shift();
    snake.push(head);
    sendMessage(`{"type": "snakePosition","clientID": "${clientID}","snake": "${convert3dToString(snake)}"}`, gameTopic);
}

function detectEnd(){
    let gameFinished:boolean = false;
    for(let i:number = 0; i < snake.length; i++){
        let snake1:number[] = snake[i];
        for (let x:number = i + 1; x < snake.length; x++){
            let snake2:number[] = snake[x]
            if(snake1[0] == snake2[0] && snake1[1] == snake2[1])
                gameFinished = true
        }
        if(snake1[0] < 0 || snake1[0] > fieldSize || snake1[1] < 0 || snake1[1] > fieldSize)
            gameFinished = true
    }
    let head:number[] = snake[snake.length - 1];
    for (let x:number = 0; x < snakePartner.length; x++){
        let snake2:number[] = snakePartner[x]
        if(head[0] == snake2[0] && head[1] == snake2[1])
            gameFinished = true
    }

    if(gameFinished){
        sendMessage(`{"type": "gameOver","loser": "${clientID}"}`, gameTopic);
        let hs:string = "";
        if(highscore < score) {
            localStorage.setItem("highscore", String(score));
            hs = "\nThis is your new highscore!!!";
        }
        Swal.fire(
            'Game Over',
            "you lost with " + score + " points." + hs + "\nTo start a new game, please reload the page.",
            'error'
        );
        clearInterval(mainLoop);
    }
}

function setupMQTT(){
    clientID = makeid(5)
    client = new Paho.MQTT.Client("wss://mqtt.eclipseprojects.io/mqtt", clientID);
    // set callback handlers
    client.onConnectionLost = function () {
        Swal.fire(
            'Server connection losr',
            "We lost the connection to the server, please reload the page.",
            'error'
        );
    }

    client.onMessageArrived = function (message:any) {
        message =  message.payloadString;
        console.log("recived: " + message)
        if (IsJsonString(message)){
            let recivedJson = JSON.parse(message);
            if(recivedJson["type"] == "newGame" && gameID == ""){
                Swal.fire({
                    title: 'New Game Invite',
                    text: `Recived new game with id ${recivedJson["gameID"]}, do you want to join?`,
                    icon: 'question',
                    showCancelButton: true,
                    confirmButtonColor: '#78e33b',
                    cancelButtonColor: '#d33',
                    confirmButtonText: 'Join'
                }).then((result:any) => {
                    if (result.isConfirmed) {
                    gameID = recivedJson["gameID"];
                    gameTopic = "/jobrouter/snakeMultiplayer/" + gameID;
                    client.subscribe(gameTopic);
                    client.unsubscribe("/jobrouter/snakeMultiplayer/games");
                    sendMessage(`{"type": "gameAccepted","gameID": "${gameID}"}`, gameTopic);
                    snake = [[fieldSize, fieldSize], [fieldSize, fieldSize - 1], [fieldSize, fieldSize - 2]];
                    }
                })
            }
            else if(recivedJson["type"] == "gameAccepted" && recivedJson["gameID"] == gameID && gameMaster){
                console.log("Game accepted");
                target = [getRandomInt(fieldSize), getRandomInt(fieldSize)];
                sendMessage(`{"type": "target","value": "${target.join(';')}"}`, gameTopic);
                Swal.fire(
                    'Game Accepted',
                    `Your game with ID ${gameID} was accepted by a player. You can start the game with any button press.`,
                    'success'
                )
                drawField();
                gameAccepted = true;
            }
            else if(recivedJson["type"] == "target"){
                console.log("recived new target");
                target = recivedJson["value"].split(";");
                drawField();
            } else if(recivedJson["type"] == "startGame" && !gameMaster && !firstButtonPressed){
                snakedir = 3;
                firstButtonPressed = true
            }
            else if(recivedJson["type"] == "gameOver"){
                if(recivedJson["loser"] != clientID){
                    let hs:string = "";
                    if(highscore < score) {
                        localStorage.setItem("highscore", String(score));
                        hs = "\nThis is your new highscore!!!";
                    }
                    Swal.fire(
                        'Game Over',
                        "you won with " + score + " points." + hs + "\nTo start a new game, please reload the page.",
                        'success'
                    );
                    clearInterval(mainLoop);
                }
            }
            else if(recivedJson["type"] == "snakePosition" && recivedJson["clientID"] != clientID){
                console.log("recived partner snake");
                snakePartner = convertStringTo3D(recivedJson["snake"]);
                drawField();
            }
            else if(recivedJson["type"] == "score" && recivedJson["clientID"] != clientID){
                partnerScoreField.innerText = recivedJson["value"];
            }
        }
    }

    console.log("connecting to broker");
    // Connect the client, providing an onConnect callback
    client.connect({
        onSuccess: onConnect
    });
}

newGameButton.onclick = function (){
    gameID = makeid(20);
    console.log("generated new gameID: " + gameID);
    gameTopic = "/jobrouter/snakeMultiplayer/" + gameID;
    sendMessage(`{"type": "newGame","gameID": "${gameID}"}`, "/jobrouter/snakeMultiplayer/games");
    client.subscribe(gameTopic);
    client.unsubscribe("/jobrouter/snakeMultiplayer/games");
    Swal.fire(
        'Game Created',
        `Your game with ID ${gameID} was created and published to the network.`,
        'success'
    )
    gameMaster = true;
    snake = [[1, 1], [1, 2], [1, 3]];

}

// Called when the connection is made
function onConnect() {
    console.log("Connected!");
    client.subscribe("/jobrouter/snakeMultiplayer/games");
}

let mainLoop = window.setInterval(function(){
    if(firstButtonPressed) {
        for (let y: number = 0; y < snake.length; y++) {
            let currentSnake = snake[y];
            if (currentSnake[0] == target[0] && currentSnake[1] == target[1]) {
                score++;
                scoreField.innerText = String(score);
                target = [getRandomInt(fieldSize), getRandomInt(fieldSize)];
                sendMessage(`{"type": "target","value": "${target.join(';')}"}`, gameTopic);
                sendMessage(`{"type": "score","value": ${score}, "clientID":"${clientID}"}`, gameTopic);
            }
        }
        moveSnake();
        detectEnd()
        drawField();
    }
}, 600);

//keypress handler
document.addEventListener('keydown', function(event) {
    if(gameMaster && !firstButtonPressed) {
        sendMessage(`{"type": "startGame"}`, gameTopic);
        firstButtonPressed = true;
    }
    const key:string = event.key;
    switch (key){
        case "ArrowRight":
            if(snakedir != 3)
                snakedir = 2
            break;
        case "ArrowLeft":
            if(snakedir != 2)
                snakedir = 3;
            break;
        case "ArrowUp":
            if(snakedir != 0)
                snakedir = 1;
            break;
        case "ArrowDown":
            if(snakedir != 1)
                snakedir = 0
    }
});

highscoreField.innerText = String(highscore);

drawField();
setupMQTT();


function makeid(length:number) {
    let result           = '';
    let characters       = 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789';
    let charactersLength = characters.length;
    for ( let i = 0; i < length; i++ ) {
        result += characters.charAt(Math.floor(Math.random() * charactersLength));
    }
    return result;
}

function IsJsonString(str:string) {
    try {
        JSON.parse(str);
    } catch (e) {
        return false;
    }
    return true;
}

function sendMessage(message:string, topic:string){
    let message_ = new Paho.MQTT.Message(message);
    message_.destinationName = topic;
    message_.qos = 0;
    client.send(message_);
}

function convert3dToString(array:number[][]){
    let tempArr:string[] = [];
    for(let i:number = 0; i < array.length; i++){
        tempArr.push(array[i].join(";"));
    }
    return tempArr.join("#");
}
function convertStringTo3D(text:string){
    let tempArr:number[][] = [];
    let snakeElements:string[] = text.split("#");
    for(let i:number = 0; i < snakeElements.length; i++){
        let coordinates:string[] = snakeElements[i].split(";");
        tempArr.push([Number(coordinates[0]), Number(coordinates[1])]);
    }
    return tempArr
}